@@ -2,8 +2,10 @@
 
 import urllib.parse
 import difflib
+import asyncio
 
 import requests
+import httpx
 import wsgiadapter
 import lxml.html
 from lxml.html import builder as E
@@ -52,14 +54,15 @@ def print_quick_pprint_diff(item1, item2):
         print(line)
 
 
-def parse_html(html):
+def parse_html(html, strict=False):
     result = lxml.html.fromstring(html)
-    output = lxml.html.tostring(result).decode("utf-8")
-    try:
-        assert output.strip() == html.strip()
-    except AssertionError:
-        print_quick_pprint_diff(html, output)
-        raise
+    if strict:
+        output = lxml.html.tostring(result).decode("utf-8")
+        try:
+            assert output.strip() == html.strip()
+        except AssertionError:
+            print_quick_pprint_diff(html, output)
+            raise
     return result
 
 
@@ -96,18 +99,39 @@ class BaseAgent(object):
 
 
 class WSGIAgent(BaseAgent):
-    def __init__(self, wsgi_app, base_url="http://example.com/"):
+    def __init__(self, wsgi_app, base_url="http://example.com/", strict=False):
         super(WSGIAgent, self).__init__(base_url)
         self.app = wsgi_app
         self.session.mount(self.base_url, wsgiadapter.WSGIAdapter(self.app))
+        self.strict = strict
+
+
+def _sync(coroutine):
+    return asyncio.get_event_loop().run_until_complete(coroutine)
+
+
+class ASGIAgent():
+    def __init__(self, app, base_url="http://example.com/"):
+        self.client = httpx.AsyncClient(app=app)
+        self.base_url = base_url
+
+    def __getattr__(self, attr):
+        response = getattr(self.client, attr)
+        return response
+
+    def get(self, url, *args, **kwargs):
+        url = urllib.parse.urljoin(self.base_url, url)
+        response = _sync(self.client.get(url, *args, **kwargs))
+        return Response(response=response, agent=self)
 
 
 class Response(object):
     def __init__(self, response, agent):
         self.response = response
         self.agent = agent
+        self.strict = getattr(agent, 'strict', False)
         if self.mime_type == "text/html" and self.content:
-            self.lxml = parse_html(self.content)
+            self.lxml = parse_html(self.content, strict=self.strict)
         else:
             self.lxml = None
 
